@@ -1,3 +1,8 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
+/*
+ * Copyright (c) 2015 Terry Cain <terrys-home.co.uk>
+ */
+
 #ifndef DRIVER_RAZERCHROMACOMMON_H_
 #define DRIVER_RAZERCHROMACOMMON_H_
 
@@ -93,6 +98,10 @@ struct razer_report razer_chroma_mouse_extended_matrix_effect_breathing_dual(uns
  */
 struct razer_report razer_chroma_misc_fn_key_toggle(unsigned char state);
 
+struct razer_report razer_chroma_misc_set_keyswitch_optimization_command1(unsigned char optimization_mode);
+struct razer_report razer_chroma_misc_set_keyswitch_optimization_command2(unsigned char optimization_mode);
+struct razer_report razer_chroma_misc_get_keyswitch_optimization(void);
+
 struct razer_report razer_chroma_misc_set_blade_brightness(unsigned char brightness);
 struct razer_report razer_chroma_misc_get_blade_brightness(void);
 
