@@ -1,14 +1,8 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * Copyright (c) 2015 Terry Cain <terry@terrys-home.co.uk>
  */
 
-/*
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- */
-
 #ifndef __HID_RAZER_MOUSE_H
 #define __HID_RAZER_MOUSE_H
 
@@ -17,6 +11,7 @@
 #define USB_DEVICE_ID_RAZER_ABYSSUS_1800 0x0020
 #define USB_DEVICE_ID_RAZER_MAMBA_2012_WIRED 0x0024
 #define USB_DEVICE_ID_RAZER_MAMBA_2012_WIRELESS 0x0025
+#define USB_DEVICE_ID_RAZER_DEATHADDER_3_5G_BLACK 0x0029
 #define USB_DEVICE_ID_RAZER_NAGA_2012 0x002E
 #define USB_DEVICE_ID_RAZER_IMPERATOR 0x002F
 #define USB_DEVICE_ID_RAZER_OUROBOROS 0x0032
@@ -79,8 +74,14 @@
 #define USB_DEVICE_ID_RAZER_VIPER_8K 0x0091
 #define USB_DEVICE_ID_RAZER_OROCHI_V2_RECEIVER 0x0094
 #define USB_DEVICE_ID_RAZER_OROCHI_V2_BLUETOOTH 0x0095
+#define USB_DEVICE_ID_RAZER_NAGA_X 0x0096
 #define USB_DEVICE_ID_RAZER_DEATHADDER_ESSENTIAL_2021 0x0098
 #define USB_DEVICE_ID_RAZER_BASILISK_V3 0x0099
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V2_X_HYPERSPEED 0x009C
+#define USB_DEVICE_ID_RAZER_VIPER_V2_PRO_WIRED 0x00A5
+#define USB_DEVICE_ID_RAZER_VIPER_V2_PRO_WIRELESS 0x00A6
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V3_PRO_WIRED 0x00B6
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V3_PRO_WIRELESS 0x00B7
 
 /* Each keyboard report has 90 bytes*/
 #define RAZER_REPORT_LEN 0x5A
@@ -124,9 +125,11 @@ struct razer_mouse_device {
 
     char serial[23]; // Now storing a random serial to be used with old devices that don't support it
 
-    unsigned char orochi2011_led;
-    unsigned char orochi2011_dpi;
-    unsigned short orochi2011_poll;
+    struct {
+        unsigned char led;
+        unsigned char dpi;
+        unsigned short poll;
+    } orochi2011;
 
     // The DeathAdder 3.5G, uses OR logic so need to remember last values. Part of a 4byte payload
     struct {
