@@ -103,6 +103,19 @@ declare -A files_metadata=(
     ["tilt_hwheel"]="rw;0"
     ["tilt_repeat"]="rw;0"
     ["tilt_repeat_delay"]="rw;0"
+    ["reset_channels"]="w;1"
+    ["channel1_size"]="rw;8"
+    ["channel2_size"]="rw;8"
+    ["channel3_size"]="rw;8"
+    ["channel4_size"]="rw;8"
+    ["channel5_size"]="rw;8"
+    ["channel6_size"]="rw;8"
+    ["channel1_led_brightness"]="rw;0"
+    ["channel2_led_brightness"]="rw;0"
+    ["channel3_led_brightness"]="rw;0"
+    ["channel4_led_brightness"]="rw;0"
+    ["channel5_led_brightness"]="rw;0"
+    ["channel6_led_brightness"]="rw;0"
     ["version"]="r;1.0.0"
 )
 
@@ -119,7 +132,7 @@ byte_dpi_devices=(
 
 
 get_attr_from_create_device_file() {
-    sed -n 's/[[:space:]]\+CREATE_DEVICE_FILE(&hdev->dev, &dev_attr_\([[:lower:]_]\+\));.*/\1/p'
+    sed -n 's/[[:space:]]\+CREATE_DEVICE_FILE(&hdev->dev, &dev_attr_\([[:lower:][:digit:]_]\+\));.*/\1/p'
 }
 
 driver=$1
